@@ -236,90 +236,104 @@ class Corr:
     def matrix_symmetric(self):
         """Symmetrizes the correlator matrices on every timeslice."""
         if self.N > 1:
-            transposed = [None if (G is None) else G.T for G in self.content]
+            transposed = [None if len(list(filter(None, np.asarray(G).flatten()))) < self.N ** 2 else G.T for G in self.content]
             return 0.5 * (Corr(transposed) + self)
         if self.N == 1:
             raise Exception("Trying to symmetrize a correlator matrix, that already has N=1.")
 
-    def GEVP(self, t0, ts=None, state=0, sorted_list=None):
-        """Solve the general eigenvalue problem on the current correlator
+    def GEVP(self, t0, ts=None, sort="Eigenvalue", **kwargs):
+        r'''Solve the generalized eigenvalue problem on the correlator matrix and returns the corresponding eigenvectors.
+
+        The eigenvectors are sorted according to the descending eigenvalues, the zeroth eigenvector(s) correspond to the
+        largest eigenvalue(s). The eigenvector(s) for the individual states can be accessed via slicing
+        ```python
+        C.GEVP(t0=2)[0]  # Ground state vector(s)
+        C.GEVP(t0=2)[:3]  # Vectors for the lowest three states
+        ```
 
         Parameters
         ----------
         t0 : int
-            The time t0 for G(t)v= lambda G(t_0)v
+            The time t0 for the right hand side of the GEVP according to $G(t)v_i=\lambda_i G(t_0)v_i$
         ts : int
-            fixed time G(t_s)v= lambda G(t_0)v  if return_list=False
-            If return_list=True and sorting=Eigenvector it gives a reference point for the sorting method.
+            fixed time $G(t_s)v_i=\lambda_i G(t_0)v_i$ if sort=None.
+            If sort="Eigenvector" it gives a reference point for the sorting method.
+        sort : string
+            If this argument is set, a list of self.T vectors per state is returned. If it is set to None, only one vector is returned.
+            - "Eigenvalue": The eigenvector is chosen according to which eigenvalue it belongs individually on every timeslice.
+            - "Eigenvector": Use the method described in arXiv:2004.10472 to find the set of v(t) belonging to the state.
+              The reference state is identified by its eigenvalue at $t=t_s$.
+
+        Other Parameters
+        ----------------
         state : int
-            The state one is interested in ordered by energy. The lowest state is zero.
-        sorted_list : string
-            if this argument is set, a list of vectors (len=self.T) is returned. If it is left as None, only one vector is returned.
-             "Eigenvalue"  -  The eigenvector is chosen according to which eigenvalue it belongs individually on every timeslice.
-             "Eigenvector" -  Use the method described in arXiv:2004.10472 [hep-lat] to find the set of v(t) belonging to the state.
-                              The reference state is identified by its eigenvalue at t=ts
-        """
-        if sorted_list is None:
+           Returns only the vector(s) for a specified state. The lowest state is zero.
+        '''
+
+        if self.N == 1:
+            raise Exception("GEVP methods only works on correlator matrices and not single correlators.")
+        if ts is not None:
+            if (ts <= t0):
+                raise Exception("ts has to be larger than t0.")
+
+        if "sorted_list" in kwargs:
+            warnings.warn("Argument 'sorted_list' is deprecated, use 'sort' instead.", DeprecationWarning)
+            sort = kwargs.get("sorted_list")
+
+        symmetric_corr = self.matrix_symmetric()
+        if sort is None:
             if (ts is None):
-                raise Exception("ts is required if sorted_list=None")
+                raise Exception("ts is required if sort=None.")
             if (self.content[t0] is None) or (self.content[ts] is None):
-                raise Exception("Corr not defined at t0/ts")
+                raise Exception("Corr not defined at t0/ts.")
             G0, Gt = np.empty([self.N, self.N], dtype="double"), np.empty([self.N, self.N], dtype="double")
-            symmetric_corr = self.matrix_symmetric()
             for i in range(self.N):
                 for j in range(self.N):
-                    G0[i, j] = symmetric_corr.content[t0][i, j].value
+                    G0[i, j] = symmetric_corr[t0][i, j].value
                     Gt[i, j] = symmetric_corr[ts][i, j].value
 
-            sp_vecs = _GEVP_solver(Gt, G0)
-            sp_vec = sp_vecs[state]
-            return sp_vec
-        else:
+            reordered_vecs = _GEVP_solver(Gt, G0)
 
-            all_vecs = []
-            for t in range(self.T):
+        elif sort in ["Eigenvalue", "Eigenvector"]:
+            if sort == "Eigenvalue" and ts is not None:
+                warnings.warn("ts has no effect when sorting by eigenvalue is chosen.", RuntimeWarning)
+            all_vecs = [None] * (t0 + 1)
+            for t in range(t0 + 1, self.T):
                 try:
                     G0, Gt = np.empty([self.N, self.N], dtype="double"), np.empty([self.N, self.N], dtype="double")
                     for i in range(self.N):
                         for j in range(self.N):
-                            G0[i, j] = self.content[t0][i, j].value
-                            Gt[i, j] = self.content[t][i, j].value
+                            G0[i, j] = symmetric_corr[t0][i, j].value
+                            Gt[i, j] = symmetric_corr[t][i, j].value
 
-                    sp_vecs = _GEVP_solver(Gt, G0)
-                    if sorted_list == "Eigenvalue":
-                        sp_vec = sp_vecs[state]
-                        all_vecs.append(sp_vec)
-                    else:
-                        all_vecs.append(sp_vecs)
+                    all_vecs.append(_GEVP_solver(Gt, G0))
                 except Exception:
                     all_vecs.append(None)
-            if sorted_list == "Eigenvector":
+            if sort == "Eigenvector":
                 if (ts is None):
                     raise Exception("ts is required for the Eigenvector sorting method.")
                 all_vecs = _sort_vectors(all_vecs, ts)
-                all_vecs = [a[state] for a in all_vecs]
 
-        return all_vecs
+            reordered_vecs = [[v[s] if v is not None else None for v in all_vecs] for s in range(self.N)]
+        else:
+            raise Exception("Unkown value for 'sort'.")
 
-    def Eigenvalue(self, t0, ts=None, state=0, sorted_list=None):
+        if "state" in kwargs:
+            return reordered_vecs[kwargs.get("state")]
+        else:
+            return reordered_vecs
+
+    def Eigenvalue(self, t0, ts=None, state=0, sort="Eigenvalue"):
         """Determines the eigenvalue of the GEVP by solving and projecting the correlator
 
         Parameters
         ----------
-        t0 : int
-            The time t0 for G(t)v= lambda G(t_0)v
-        ts : int
-            fixed time G(t_s)v= lambda G(t_0)v  if return_list=False
-            If return_list=True and sorting=Eigenvector it gives a reference point for the sorting method.
         state : int
             The state one is interested in ordered by energy. The lowest state is zero.
-        sorted_list : string
-            if this argument is set, a list of vectors (len=self.T) is returned. If it is left as None, only one vector is returned.
-             "Eigenvalue"  -  The eigenvector is chosen according to which eigenvalue it belongs individually on every timeslice.
-             "Eigenvector" -  Use the method described in arXiv:2004.10472 [hep-lat] to find the set of v(t) belonging to the state.
-                              The reference state is identified by its eigenvalue at t=ts
+
+        All other parameters are identical to the ones of Corr.GEVP.
         """
-        vec = self.GEVP(t0, ts=ts, state=state, sorted_list=sorted_list)
+        vec = self.GEVP(t0, ts=ts, sort=sort)[state]
         return self.projected(vec)
 
     def Hankel(self, N, periodic=False):
@@ -621,7 +635,7 @@ class Corr:
             raise Exception('Unknown variant.')
 
     def fit(self, function, fitrange=None, silent=False, **kwargs):
-        """Fits function to the data
+        r'''Fits function to the data
 
         Parameters
         ----------
@@ -629,10 +643,12 @@ class Corr:
             function to fit to the data. See fits.least_squares for details.
         fitrange : list
             Two element list containing the timeslices on which the fit is supposed to start and stop.
+            Caution: This range is inclusive as opposed to standard python indexing.
+            `fitrange=[4, 6]` corresponds to the three entries 4, 5 and 6.
             If not specified, self.prange or all timeslices are used.
         silent : bool
             Decides whether output is printed to the standard output.
-        """
+        '''
         if self.N != 1:
             raise Exception("Correlator must be projected before fitting")
 
@@ -701,7 +717,7 @@ class Corr:
         self.prange = prange
         return
 
-    def show(self, x_range=None, comp=None, y_range=None, logscale=False, plateau=None, fit_res=None, ylabel=None, save=None, auto_gamma=False):
+    def show(self, x_range=None, comp=None, y_range=None, logscale=False, plateau=None, fit_res=None, ylabel=None, save=None, auto_gamma=False, hide_sigma=None, references=None):
         """Plots the correlator using the tag of the correlator as label if available.
 
         Parameters
@@ -723,6 +739,10 @@ class Corr:
             path to file in which the figure should be saved
         auto_gamma : bool
             Apply the gamma method with standard parameters to all correlators and plateau values before plotting.
+        hide_sigma : float
+            Hides data points from the first value on which is consistent with zero within 'hide_sigma' standard errors.
+        references : list
+            List of floating point values that are displayed as horizontal lines for reference.
         """
         if self.N != 1:
             raise Exception("Correlator must be projected before plotting")
@@ -737,7 +757,11 @@ class Corr:
         ax1 = fig.add_subplot(111)
 
         x, y, y_err = self.plottable()
-        ax1.errorbar(x, y, y_err, label=self.tag)
+        if hide_sigma:
+            hide_from = np.argmax((hide_sigma * np.array(y_err[1:])) > np.abs(y[1:])) - 1
+        else:
+            hide_from = None
+        ax1.errorbar(x[:hide_from], y[:hide_from], y_err[:hide_from], label=self.tag)
         if logscale:
             ax1.set_yscale('log')
         else:
@@ -756,7 +780,11 @@ class Corr:
                     if auto_gamma:
                         corr.gamma_method()
                     x, y, y_err = corr.plottable()
-                    plt.errorbar(x, y, y_err, label=corr.tag, mfc=plt.rcParams['axes.facecolor'])
+                    if hide_sigma:
+                        hide_from = np.argmax((hide_sigma * np.array(y_err[1:])) > np.abs(y[1:])) - 1
+                    else:
+                        hide_from = None
+                    plt.errorbar(x[:hide_from], y[:hide_from], y_err[:hide_from], label=corr.tag, mfc=plt.rcParams['axes.facecolor'])
             else:
                 raise Exception("'comp' must be a correlator or a list of correlators.")
 
@@ -768,6 +796,14 @@ class Corr:
                 ax1.axhspan(plateau.value - plateau.dvalue, plateau.value + plateau.dvalue, alpha=0.25, color=plt.rcParams['text.color'], ls='-')
             else:
                 raise Exception("'plateau' must be an Obs")
+
+        if references:
+            if isinstance(references, list):
+                for ref in references:
+                    ax1.axhline(y=ref, linewidth=1, color=plt.rcParams['text.color'], alpha=0.6, marker=',', ls='--')
+            else:
+                raise Exception("'references' must be a list of floating pint values.")
+
         if self.prange:
             ax1.axvline(self.prange[0], 0, 1, ls='-', marker=',')
             ax1.axvline(self.prange[1], 0, 1, ls='-', marker=',')
@@ -1101,6 +1137,63 @@ class Corr:
 
         return self._apply_func_to_corr(return_imag)
 
+    def prune(self, Ntrunc, tproj=3, t0proj=2, basematrix=None):
+        r''' Project large correlation matrix to lowest states
+
+        This method can be used to reduce the size of an (N x N) correlation matrix
+        to (Ntrunc x Ntrunc) by solving a GEVP at very early times where the noise
+        is still small.
+
+        Parameters
+        ----------
+        Ntrunc: int
+            Rank of the target matrix.
+        tproj: int
+            Time where the eigenvectors are evaluated, corresponds to ts in the GEVP method.
+            The default value is 3.
+        t0proj: int
+            Time where the correlation matrix is inverted. Choosing t0proj=1 is strongly
+            discouraged for O(a) improved theories, since the correctness of the procedure
+            cannot be granted in this case. The default value is 2.
+        basematrix : Corr
+            Correlation matrix that is used to determine the eigenvectors of the
+            lowest states based on a GEVP. basematrix is taken to be the Corr itself if
+            is is not specified.
+
+        Notes
+        -----
+        We have the basematrix $C(t)$ and the target matrix $G(t)$. We start by solving
+        the GEVP $$C(t) v_n(t, t_0) = \lambda_n(t, t_0) C(t_0) v_n(t, t_0)$$ where $t \equiv t_\mathrm{proj}$
+        and $t_0 \equiv t_{0, \mathrm{proj}}$. The target matrix is projected onto the subspace of the
+        resulting eigenvectors $v_n, n=1,\dots,N_\mathrm{trunc}$ via
+        $$G^\prime_{i, j}(t) = (v_i, G(t) v_j)$$. This allows to reduce the size of a large
+        correlation matrix and to remove some noise that is added by irrelevant operators.
+        This may allow to use the GEVP on $G(t)$ at late times such that the theoretically motivated
+        bound $t_0 \leq t/2$ holds, since the condition number of $G(t)$ is decreased, compared to $C(t)$.
+        '''
+
+        if self.N == 1:
+            raise Exception('Method cannot be applied to one-dimensional correlators.')
+        if basematrix is None:
+            basematrix = self
+        if Ntrunc >= basematrix.N:
+            raise Exception('Cannot truncate using Ntrunc <= %d' % (basematrix.N))
+        if basematrix.N != self.N:
+            raise Exception('basematrix and targetmatrix have to be of the same size.')
+
+        evecs = basematrix.GEVP(t0proj, tproj, sort=None)[:Ntrunc]
+
+        tmpmat = np.empty((Ntrunc, Ntrunc), dtype=object)
+        rmat = []
+        for t in range(basematrix.T):
+            for i in range(Ntrunc):
+                for j in range(Ntrunc):
+                    tmpmat[i][j] = evecs[i].T @ self[t] @ evecs[j]
+            rmat.append(np.copy(tmpmat))
+
+        newcontent = [None if (self.content[t] is None) else rmat[t] for t in range(self.T)]
+        return Corr(newcontent)
+
 
 def _sort_vectors(vec_set, ts):
     """Helper function used to find a set of Eigenvectors consistent over all timeslices"""
@@ -1129,8 +1222,10 @@ def _sort_vectors(vec_set, ts):
     return sorted_vec_set
 
 
-def _GEVP_solver(Gt, G0):  # Just so normalization an sorting does not need to be repeated. Here we could later put in some checks
-    sp_val, sp_vecs = scipy.linalg.eigh(Gt, G0)
-    sp_vecs = [sp_vecs[:, np.argsort(sp_val)[-i]] for i in range(1, sp_vecs.shape[0] + 1)]
-    sp_vecs = [v / np.sqrt((v.T @ G0 @ v)) for v in sp_vecs]
-    return sp_vecs
+def _GEVP_solver(Gt, G0):
+    """Helper function for solving the GEVP and sorting the eigenvectors.
+
+    The helper function assumes that both provided matrices are symmetric and
+    only processes the lower triangular part of both matrices. In case the matrices
+    are not symmetric the upper triangular parts are effectively discarded."""
+    return scipy.linalg.eigh(Gt, G0, lower=True)[1].T[::-1]
